/*!
 * vue-masonry-css v0.0.1
 * https://github.com/paulcollett/vue-masonry-css
 * Released under the MIT License.
 */

var MasonryComponent = {
  props: {
    tag: {
      type: [String],
      default: 'div'
    },
    cols: {
      type: [Object, Number],
      default: 2
    },
    gutter: {
      type: [Object, Number, String],
      default: 2
    },
    css: {
      type: [Boolean],
      default: true
    }
  },
  data: function() {
    return {
      displayColumns: 2,
      displayGutter: '0px'
    }
  },
  mounted: function() {
    this.reCalculate();

    if(window) {
      window.addEventListener('resize', this.reCalculate);
    }
  },
  beforeDestroy: function() {
    if(window) {
      window.removeEventListener('resize', this.reCalculate);
    }
  },
  methods: {
    reCalculate: function() {
      var windowWidth = (window ? window.innerWidth : null) || Infinity;

      this.reCalculateColumnCount(windowWidth);

      this.reCalculateGutterSize(windowWidth);
    },
    _getBreakpointValue: function(mixed, windowWidth) {
      var valueAsNum = parseInt(mixed);

      if(valueAsNum > -1) {
        return mixed;
      }else if(typeof mixed !== 'object') {
        return 0;
      }

      var matchedBreakpoint = Infinity;
      var matchedValue = mixed.default || 0;

      for(var k in mixed) {
        var breakpoint = parseInt(k);
        var breakpointValRaw = mixed[breakpoint];
        var breakpointVal = parseInt(breakpointValRaw);

        if(isNaN(breakpoint) || isNaN(breakpointVal)) {
          continue;
        }

        var isNewBreakpoint = windowWidth <= breakpoint && breakpoint < matchedBreakpoint;

        if(isNewBreakpoint) {
          matchedBreakpoint = breakpoint;
          matchedValue = breakpointValRaw;
        }
      }

      return matchedValue;
    },
    reCalculateGutterSize: function(windowWidth) {
      this.displayGutter = this._getBreakpointValue(this.gutter, windowWidth);
    },
    reCalculateColumnCount: function(windowWidth) {
      var newColumns = this._getBreakpointValue(this.cols, windowWidth);

      // final bit of making sure its a correct value
      newColumns = Math.max(1, newColumns * 1 || 0);

      this.displayColumns = newColumns;
    },
    itemsInColumns: function() {
      var currentColumnCount = this.displayColumns;
      var itemsInColumns = [];
      var items = this.$slots.default || [];

      // This component does not work with a child
      // <transition-group /> ..yet, so we'll be helpful and skip it
      if(items.length === 1 && items[0].componentOptions && items[0].componentOptions.tag == 'transition-group') {
        items = items[0].componentOptions.children;
      }

      for (var i = 0; i < items.length; i++) {
        // skip Vues' text tags
        if(!items[i].tag && items[i].text) {
          i--;
          items.splice(i, 1);
          continue;
        }

        var columnIndex = i % currentColumnCount;

        if(!itemsInColumns[columnIndex]) {
          itemsInColumns[columnIndex] = [];
        }

        itemsInColumns[columnIndex].push(items[i]);
      }

      return itemsInColumns;
    }
  },
  render: function (createElement) {
    var this$1 = this;

    var childrenInColumns = this.itemsInColumns();
    var columns = [];

    for(var i = 0; i < childrenInColumns.length; i++) {
      var column = createElement('div', {
        class: 'hello',
        key: i + '-' + childrenInColumns.length,
        style: {
          boxSizing: 'border-box',
          backgroundClip: 'padding-box',
          width: (100 / this$1.displayColumns) + '%',
          border: '0 solid transparent',
          borderLeftWidth: this$1.displayGutter
        }
      }, childrenInColumns[i]);

      columns.push(column);
    }

    this.prevColumns = childrenInColumns;

    var wrapper = createElement(
      this.tag,   // tag name
      this.css ? {
        //class: 'my-mason',
        style: {
          display: 'flex',
          marginLeft: '-' + this.displayGutter
        }
      } : null,
      columns // array of children
    );

    return wrapper;
  }
};

var Plugin = function () {};

Plugin.install = function (Vue, options) {
  if (Plugin.installed) {
    return;
  }

  Vue.component((options && options.name) || 'masonry', MasonryComponent);
};

if (typeof window !== 'undefined' && window.Vue) {
  window.Vue.use(Plugin);
}

export default Plugin;
